//  COM Support Classes
//  Copyright  1996
//  B. W. Bush
//  All rights reserved
//
//  $RCSfile: IClassFactory.cpp $
//  $Revision: 1.0 $
//  $State: Rel $
//  $Date: 1996/04/29 18:54:59 $


//  Include windows header files.
#include <windows>
#include <objbase>

//  Include COM support classes header files.
#include <com\IClassFactory.h>

//  Create an instance.
TIClassFactory::TIClassFactory(const CLSID& clsid)
  : fReferences(0),
    fClsid(clsid)
{
    TIClassFactory::IncrementObjectCount();
}

//  Destroy an instance.
TIClassFactory::~TIClassFactory()
{
    TIClassFactory::DecrementObjectCount();
}

//  Return whether there are any associated locks.
bool TIClassFactory::HasLocks()
{
    return fgLockCount != 0;
}

//  Return whether there are any associated objects.
bool TIClassFactory::HasObjects()
{
    return fgObjectCount != 0;
}

//  Increment the number of associated objects.
ULONG TIClassFactory::IncrementObjectCount()
{
    return ++fgObjectCount;
}

//  Decrement the number of associated objects.
ULONG TIClassFactory::DecrementObjectCount()
{
    const ULONG count = --fgObjectCount;
    if (count == 0 && fgCallback != NULL)
        (*fgCallback)();
    return count;
}

//  Set the callback for notification when there are no associated objects.
void TIClassFactory::SetCallback(EmptyNotifier FAR* callback)
{
    fgCallback = callback;
}

//  Query the implementation.
HRESULT STDMETHODCALLTYPE TIClassFactory::QueryInterface(REFIID iid, LPVOID FAR* iface)
{
    try {
        if (IsEqualIID(iid, IID_IUnknown)) {
            const LPUNKNOWN unknown = (LPUNKNOWN) this;
            unknown->AddRef();
            *iface = unknown;
        } else if (IsEqualIID(iid, IID_IClassFactory)) {
            const LPCLASSFACTORY factory = (LPCLASSFACTORY) this;
            factory->AddRef();
            *iface = factory;
        } else
            throw E_NOINTERFACE;
    } catch(const HRESULT result) {
        *iface = NULL;
        return result;
    }
    return S_OK;
}

//  Add a reference to the implemenation.
ULONG STDMETHODCALLTYPE TIClassFactory::AddRef()
{
    return ++fReferences;
}

//  Release a reference from the implemenation.
ULONG STDMETHODCALLTYPE TIClassFactory::Release()
{
    const ULONG references = --fReferences;
    if (fReferences == 0)
        delete this;
    return references;
}

//  Add a lock to the server.
HRESULT STDMETHODCALLTYPE TIClassFactory::LockServer(BOOL lock)
{
    HRESULT result;
    if (lock) {
        ++fgLockCount;
        result = S_OK;
    } else if (fgLockCount > 0) {
        --fgLockCount;
        result = S_OK;
    } else
        result = E_UNEXPECTED;
    return result;
}

//  The factories have a lock count.
ULONG TIClassFactory::fgLockCount = 0;

//  The factories have an associated object count.
ULONG TIClassFactory::fgObjectCount = 0;

//  The factories have a callback function to notify when there are no objects.
TIClassFactory::EmptyNotifier FAR* TIClassFactory::fgCallback;

