//  COM Support Classes
//  Copyright  1996
//  B. W. Bush
//  All rights reserved
//
//  $RCSfile: IClassFactory.h $
//  $Revision: 1.0 $
//  $State: Rel $
//  $Date: 1996/04/29 18:54:59 $


#ifndef BWB_IClassFactory
#define BWB_IClassFactory


//  Include windows header files.
#include <windows>
#include <objbase>

//  A class factory constructs objects.
class TIClassFactory
  : public IClassFactory
{
  public:

    //  Type definitions.
    typedef void EmptyNotifier();

    //  Construct an instance.
    TIClassFactory(const CLSID& clsid);

    //  Return whether there are any associated locks.
    static bool HasLocks();

    //  Return whether there are any associated objects.
    static bool HasObjects();

    //  Increment the number of associated objects.
    static ULONG IncrementObjectCount();

    //  Decrement the number of associated objects.
    static ULONG DecrementObjectCount();

    //  Set the callback for notification when there are no associated objects.
    static void SetCallback(EmptyNotifier FAR* callback);

    //  Query the implementation.
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID FAR* iface);

    //  Add a reference to the implemenation.
    ULONG STDMETHODCALLTYPE AddRef();

    //  Release a reference from the implemenation.
    ULONG STDMETHODCALLTYPE Release();

    //  Add a lock to the server.
    HRESULT STDMETHODCALLTYPE LockServer(BOOL);

  protected:

    //  Destroy an instance.
    ~TIClassFactory();

    //  Each instance is associated with a particular class.
    CLSID fClsid;

  private:

    //  Each instance is reference counted.
    ULONG fReferences;

    //  The factories have a lock count.
    static ULONG fgLockCount;

    //  The factories have an associated object count.
    static ULONG fgObjectCount;

    //  The factories have a callback function to notify when there are no objects.
    static EmptyNotifier FAR* fgCallback;
};


#endif // BWB_IClassFactory

