//  COM Support Classes
//  Copyright  1996
//  B. W. Bush
//  All rights reserved
//
//  $RCSfile: InterfacePointer.h $
//  $Revision: 1.0 $
//  $State: Rel $
//  $Date: 1996/04/29 18:54:59 $


#ifndef BWB_InterfacePointer
#define BWB_InterfacePointer


//  A wrapper for a COM interface pointer of the given type with the specified interface.
template <class AType, const IID* AnInterface = &IID_IUnknown>
class TInterfacePointer
{
  public:

    //  Construct an empty instance.
    TInterfacePointer()
      : fPointer(NULL)
    {
    }

    //  Construct an instance for the specified class id with the given contest.
    TInterfacePointer(REFIID clsid, DWORD context = CLSCTX_INPROC_SERVER)
      : fPointer(NULL)
    {
        IClassFactory* factory;
        CoGetClassObject(clsid, context, NULL, IID_IClassFactory, (LPVOID FAR*) &factory);
        factory->CreateInstance(NULL, *AnInterface, (LPVOID FAR*) &fPointer);
        factory->Release();
    }

    //  Construct an instance for the specified pointer.
    TInterfacePointer(AType* pointer)
      : fPointer(pointer)
    {
    }

    //  Construct a copy of the specified instance.
    TInterfacePointer(const TInterfacePointer<AType, AnInterface>& pointer)
      : fPointer(pointer)
    {
        if (fPointer != NULL)
            fPointer->AddRef();
    }

    //  Destroy an instance.
    ~TInterfacePointer()
    {
        if (fPointer != NULL)
            fPointer->Release();
    }

    //  Return the interface pointer.
    operator AType*() const
    {
        return fPointer;
    }

    //  Copy the specified instance.
    TInterfacePointer<AType, AnInterface>& operator=(const TInterfacePointer<AType, AnInterface>& pointer)
    {
        if ((AType*)(pointer) != NULL)
            pointer->AddRef();
        if (fPointer != NULL)
            fPointer->Release();
        fPointer = pointer;
        return *this;
    }

    //  Copy the specified interface pointer.
    TInterfacePointer<AType, AnInterface>& operator=(AType* pointer)
    {
        if (fPointer != NULL)
            fPointer->Release();
        fPointer = pointer;
        return *this;
    }

    //  Dereference the interface pointer.
    AType& operator*() const
    {
        return *fPointer;
    }

    //  Dereference the interface pointer.
    AType* operator->() const
    {
        return fPointer;
    }

  private:

    //  Wrapped interface pointer.
    AType* fPointer;
};


#endif // BWB_InterfacePointer

